#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File  : DQSurv.py
# Author: My Duan
# Date  : 2023-02-15

from preprocess import *
from options import parse_args
from evaluate import *
from model_mtl import myModel

opt = parse_args()

def get_data(data, survival, driver, target, missing_value = '-9.9658', log_transform=False):

	obj_data, survival_label = match_survival_data(data, survival)
	day_stratified = get_split_df(survival_label['survival'].values)
	survival_label['stratified'] = day_stratified

	print('Data shape is: {}, label shape is: {}.'.format(obj_data.shape, survival_label.shape))

	'''transform the null to np.nan'''

	'''log-transform'''

	if log_transform == True:
		import math
		print('Log transform ...')
		try:
			obj_data = obj_data.applymap(lambda x: math.log2(x + 0.001))
		except:
			print('Please check whether the data needs log transform.')

	# print('missing value is :', missing_value)

	obj_data = transform_na(obj_data, missing_value)
	'''by the driver and target genes, fill unavaliable features by 0.0'''
	print('Check and pad the unavaliabel feautres...')
	obj_data = preprocess_feats(obj_data, driver, target)

	return obj_data, survival_label


def evaluate(train_dl, valid_dl, test_dl, tr_R, va_R, te_R, input_dim, model_path, gat_layer, saved_flag):

	# print('Here! Please load the prediction values based on health model !')

	model = myModel(pretrained_model=gat_layer, input_dim=input_dim,
						hidden_dim=8, device=opt.device, dropout=0.3).to(opt.device)

	loss, ci, pred_reg = train_evaluate(train_dl, valid_dl, test_dl,
							 tr_R, va_R, te_R,
							 model, opt, model_path,
							 alpha=0.5, lr = opt.learning_rate, weight_decay = opt.weight_decay,
							 saved_flag=saved_flag)




	return loss, ci, pred_reg


def DQSurv(obj_data, survival_label, driver, target, edge_index,
		   pretrained_model_path, model_path, pretrained, saved_flag=False):

	train_df, test_df, valid_df, train_y, test_y, valid_y= split_data(obj_data, survival_label,
																	  test_size=opt.test_size)

	train_driver = train_df.loc[:, driver].values
	train_targ = train_df.loc[:, target].values

	test_driver = test_df.loc[:, driver].values
	test_targ = test_df.loc[:, target].values

	valid_driver = valid_df.loc[:,driver].values
	valid_targ = valid_df.loc[:, target].values

	print('train_driver, test_driver, valid_driver, train_targ, test_targ, valid_targ, train_y, test_y, valid_y')
	print([i.shape for i in [train_driver, test_driver, valid_driver, train_targ, test_targ, valid_targ,
							 train_y, test_y, valid_y]])

	train_dl, valid_dl, test_dl, tr_R, va_R, te_R, input_dim = \
		get_loader(train_driver, test_driver, valid_driver, train_targ, test_targ,
				   valid_targ, train_y, test_y, valid_y, edge_index=edge_index)

	test_df.loc[:, target].to_csv(model_path + '/test_target.csv')

	print('\nModeling ...')

	if pretrained == False:

		print('\nRandom Initialization version.\n')

		from torch_geometric.nn import GATConv
		gat_layer = GATConv(in_channels=1, out_channels=8)

		mse, ci, pred_targ = evaluate(train_dl, valid_dl, test_dl, tr_R, va_R, te_R, input_dim, model_path,
						   gat_layer=gat_layer, saved_flag=saved_flag)

		pd.DataFrame(pred_targ, columns=target).to_csv(model_path + '/pred_target.csv')

	elif pretrained == True:

		print('Transfer learning version.')

		pretrained_model = torch.load(pretrained_model_path).to(opt.device)
		TL_gat_layer = list(pretrained_model.modules())[1]
		TL_mse, TL_ci, TL_pred_targ = evaluate(train_dl, valid_dl, test_dl, tr_R, va_R, te_R, input_dim, model_path,
								 gat_layer = TL_gat_layer, saved_flag=saved_flag)

		pd.DataFrame(TL_pred_targ, columns=target).to_csv(model_path + '/pred_target.csv')


