#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File  : GCN_feature.py
# Author: My Duan
# Date  : 2022-08-02

import torch
import torch.nn as nn
import torch.nn.functional as F
from torch_geometric.nn import GATConv
import numpy as np

class GAT(nn.Module):
	def __init__(self, input_dim, node_num, output_dim, hidden_size, device, dropout=0.3):
		super().__init__()

		self.conv1 = GATConv(input_dim, hidden_size)
		self.linear = nn.Linear(node_num * hidden_size, output_dim)
		self.leakyrelu = nn.LeakyReLU()
		self.Dropout = nn.Dropout(p=dropout)
		self.device = device

	def global_flatten(self, x, batch):

		batch_num = np.unique(batch.cpu().numpy())
		batch_samples = []
		for i in batch_num:
			batch_index = np.where(batch.cpu().numpy() == i)
			sample = x[batch_index].reshape(1, -1)
			batch_samples.append(sample)
		pool = torch.cat(batch_samples, dim=0)
		return pool

	def forward(self, data):

		x, adj, batch = data.x.to(self.device), data.edge_index.to(self.device), data.batch.to(self.device)

		x_ = self.conv1(x, adj)
		x_ = self.leakyrelu(x_)
		
		'''graph pooling'''

		x_ = self.global_flatten(x_, batch)
		
		x_ = self.Dropout(x_)
		x_ = self.linear(x_)

		return x_




