#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File  : model_mtl.py
# Author: My Duan
# Date  : 2022-10-31

import torch
import numpy as np
import torch.nn as nn
import copy

class myModel(nn.Module):

	def __init__(self, pretrained_model, input_dim, hidden_dim, device, dropout=0.3):
		super(myModel, self).__init__()

		self.GAT = pretrained_model
		self.health_model =  copy.deepcopy(pretrained_model)

		for param in self.health_model.parameters():
			param.requires_grad = False

		self.device = device

		self.linear1 = nn.Linear(input_dim*hidden_dim , 3062)
		self.linear2 = nn.Linear(input_dim , 64)
		self.linear3 = nn.Linear(6400, 128) # [6400, 128]

		self.fc_output = nn.Linear(64*3, 1)

		self.ReLU = nn.ReLU()
		self.LeakyReLU = nn.LeakyReLU()
		self.Dropout = nn.Dropout(p=dropout)

	def global_flatten(self, x, batch):
		batch_num = np.unique(batch.cpu().numpy())
		batch_samples = []
		for i in batch_num:
			batch_index = np.where(batch.cpu().numpy() == i)
			sample = x[batch_index].reshape(1, -1)
			batch_samples.append(sample)
		#         batch_samples = np.array(batch_samples)
		pool = torch.cat(batch_samples, dim=0)
		#         return torch.tensor(batch_samples, dtype=torch.float)
		return pool


	def forward(self, data):
		# x, adj, batch = data.x.to(), data.edge_index, data.batch
		x, adj, batch = data.x.to(self.device), data.edge_index.to(self.device), data.batch.to(self.device)
		batch_num = np.unique(batch.cpu().numpy()).shape[0]
		x0 = x.reshape(batch_num, -1)
		x0 = self.Dropout(x0)

		bottom = self.GAT(x, adj)

		bottom = self.global_flatten(bottom, batch)
		bottom = self.Dropout(bottom)

		targ = self.LeakyReLU(bottom)
		# targ = self.Dropout(targ)
		targ = self.linear1(targ) # 3062

		h_bottom = self.health_model(x, adj)

		h_bottom = self.global_flatten(h_bottom, batch)
		h_bottom = self.Dropout(h_bottom)

		mq_semantic = torch.sub(h_bottom, bottom) # [batch, 6400]

		x0   = self.linear2(x0) # 800->64
		x0   = self.ReLU(x0)

		mq_x = self.linear3(mq_semantic) # 6400 -> 128
		mq_x   = self.ReLU(mq_x)

		mq_x = torch.cat((mq_x, x0), dim=1) # 64*3=252

		x_ = self.fc_output(mq_x) # 128->1

		return targ, x_